@@ -3,6 +3,7 @@
 import argparse
 import ast
 import collections
+import importlib.metadata
 import inspect
 import itertools
 import linecache
@@ -15,10 +16,26 @@ import sys
 import typing
 import warnings
 
-# Ignore, at least, these directory basenames (default directories to ignore).
-default_ignore_dirs = (".venv", ".env", ".git", ".hg", ".pytest_cache", ".ruff_cache", "__pycache__")
+import tomllib
 
+# Ignore, at least, these directory basenames (default directories to ignore).
+default_ignore_dirs = (
+    # Common virtual environment directories
+    "venv",
+    "env",
+    ".venv",
+    ".env",
+    # Common build directories
+    ".git",  # Git
+    ".hg",  # Mercurial
+    ".pytest_cache",  # Pytest cache
+    ".ruff_cache",  # Ruff cache
+    "__pycache__",  # Python cache
+)
+
+# ============================================================================
 # Docstring sections: https://www.sphinx-doc.org/en/master/usage/domains/python.html#info-field-lists
+# ============================================================================
 ptype_key = "type"
 rtype_key = "rtype"
 param_set = {"param", "parameter", "arg", "argument", "keyword", "key"}
@@ -26,31 +43,57 @@ return_set = {"return", "returns"}
 raises_set = {"raises", "raise", "except", "exception"}
 
 # Although Sphinx does not currently explain this distinction, its syntax was directly inherited from
-# Epydoc (https://epydoc.sourceforge.net/manual-fields.html#variables),
-# so it is generally understood that:
-# - "ivar": instance variable
-# - "cvar": class variable
-# - "var": variable (global or module-level variable)
+# Epydoc (https://epydoc.sourceforge.net/manual-fields.html#variables), so it is generally understood that:
+# - ivar: instance variable
+# - cvar: class variable
+# - var: variable (global or module-level variable)
 class_var_set = {"ivar", "cvar"}
 module_var_key = "var"
 vartype_key = "vartype"
 ignore_set = {"meta"}  # At this moment, it's ignored
 
+# Sections that define types
 types_set = {ptype_key, rtype_key, vartype_key}  # Sections that define types
 
+# ============================================================================
+# Regular expressions for parsing docstrings:
+# ============================================================================
 # Summary (everything before the first section or the end of the string)
 summary_regex = re.compile(r'^(.*?)(?=^:|\Z)', flags=re.DOTALL | re.MULTILINE)
 # Section (start with ':' and end before next ':' at start of line or end of string)
 section_regex = re.compile(r'(^:.*?)(?=^:|\Z)', flags=re.DOTALL | re.MULTILINE)
-# Trailing empty lines
+# Trailing blank lines
 trailing_regex = re.compile("(?:^\\s*$){2,}\\Z", flags=re.MULTILINE)
-# Consecutive empty lines (not at end)
-empty_lines_regex = re.compile("(?:^[ \t]*\r?\n){2,}(?=[^\r\n])", re.MULTILINE)
+# Consecutive blank lines (not at end)
+blank_lines_regex = re.compile("(?:^[ \t]*\r?\n){2,}(?=[^\r\n])", flags=re.MULTILINE)
 # Docstring starting or ending with only quotes lines
 quotes_starts_regex = re.compile(r'^"+\s*$')
 quotes_ends_regex = re.compile(r'^\s*"+$')
-# Bad whitespace in section definition: (consecutive spaces or trailing space or leading space)
-section_bad_ws_regex = re.compile(r'(\s{2,}|^\s+|\s+$)')
+# Leading whitespaces matcher (ignoring break-line)
+leading_ws_regex = re.compile('^[ \t]+')
+
+# ----------------------------------
+# Bad whitespace in section left-side:
+# ----------------------------------
+# - consecutive whitespace (anywhere)
+# - leading/trailing whitespace (ignore break-line)
+section_ws_left_err = re.compile('(\\s{2,}|^[ \t]+|[ \t]+$)').search
+
+# ----------------------------------
+# Bad whitespace in section right-side (type):
+# ----------------------------------
+# - missing leading whitespace (starts with non-space)
+# - consecutive whitespace (anywhere)
+# - trailing whitespace (ignore break-line)
+section_ws_right_type_err = re.compile('(^\\S|\\s{2,}|[ \t]+$)').search
+
+# ----------------------------------
+# Bad whitespace in section right-side (description):
+# ----------------------------------
+# - missing leading whitespace (starts with non-space)
+# - starts with 2+ whitespace (ignore break-line)
+# - trailing whitespace (ignore break-line)
+section_ws_right_desc_err = re.compile('(^\\S|^[ \t]{2,}|[ \t]+$)').search
 
 
 class NodeTypes:
@@ -59,6 +102,32 @@ class NodeTypes:
     MODULE = "module"
 
 
+# ----------------------------------------------------------------------------
+# Function definition parsed properties
+# ----------------------------------------------------------------------------
+
+
+class ParsedFuncParam(typing.NamedTuple):
+    name: str
+    order: int
+    annotation: str | None
+
+
+class ParsedFuncReturn(typing.NamedTuple):
+    has_return: bool  # True if the function has a return or yield statement.
+    annotation: str | None
+
+
+class ParsedFuncDef(typing.NamedTuple):
+    parameters: dict[str, ParsedFuncParam]
+    returns: ParsedFuncReturn
+    is_implemented: bool  # True if the function is implemented (not just a stub)
+
+
+# ----------------------------------------------------------------------------
+# Function docstring parsed properties
+# ----------------------------------------------------------------------------
+
 class ParsedDocsParam(typing.NamedTuple):
     section_key: str
     sep: bool  # True if ':' was present
@@ -108,9 +177,8 @@ class ParsedDocs(typing.NamedTuple):
     docs: str | None  # Cleaned docstring (inspect.cleandoc)
     docs_ini_lineno: int | None  # First line number of the docstring, None if no docstring
     docs_end_lineno: int | None  # End line number of the docstring, None if no docstring
-    code_ini_lineno: int | None  # First line number of the code or comment after the docstring.
-
-    non_empty_end_lines: bool  # True if there are non-empty lines after the last section
+    missing_blank_line_after: bool  # True if missing blank line after docstring
+    non_blank_end_lines: bool  # True if there are non-blank lines after the last section
 
 
 class Violations:
@@ -133,7 +201,8 @@ class Violations:
     DOC009 = (True, "DOC009", "Docstring must not use more than 3 double quotes")
 
     DOC010 = (True, "DOC010", "Section definition contains invalid whitespace ({!r})")
-    DOC011 = (True, "DOC011", "Trailing non-empty lines after last section.")
+    DOC011 = (True, "DOC011", "Trailing non-empty lines after last section")
+    DOC012 = (True, "DOC012", "Leading whitespaces in first non-blank line ({!r})")
 
     # DOC1xx: Parameter issues
     DOC101 = (True, "DOC101", "Parameter documented but not in signature ({!r})")
@@ -141,6 +210,8 @@ class Violations:
     DOC103 = (True, "DOC103", "Parameter type already in signature ({!r})")
     DOC104 = (True, "DOC104", "Parameter type mismatch with annotation ({!r} != {!r})")
     DOC105 = (True, "DOC105", "Duplicated parameter ({!r})")
+    DOC106 = (True, "DOC106", "Parameter order mismatch with signature")
+    DOC107 = (False, "DOC107", "Missing parameter in docstring ({!r})")  # Disabled by default
 
     # DOC2xx: Return issues
     DOC201 = (True, "DOC201", "Return documented but function has no return statement")
@@ -190,10 +261,12 @@ class Violations:
         return cls.is_valid_syntax(f"x: {hint}", mode="exec") if hint else False
 
     @classmethod
-    def validate_empty_lines(cls, parsed, /):
-        text = parsed.rawdocs
-        if text:
-            hits = empty_lines_regex.finditer(text)
+    def validate_blank_lines(cls, parsed_docs, /):
+        if parsed_docs.missing_blank_line_after:
+            yield cls.DOC003, ()
+
+        if text := parsed_docs.rawdocs:
+            hits = blank_lines_regex.finditer(text)
             present = next(filter(None, hits), None)
             if present:
                 yield cls.DOC005, ()
@@ -206,14 +279,28 @@ class Violations:
                     # finditer: Empty matches are included in the result.
                     yield cls.DOC006, ()
 
+        if parsed_docs.non_blank_end_lines:
+            yield cls.DOC011, ()
+
     @classmethod
-    def validate_bad_whitespaces(cls, parsed, /):
-        for section_def in parsed.bad_whitespaces_def:
+    def validate_whitespaces(cls, parsed_docs, /):
+        lines = parsed_docs.rawdocs.splitlines() if parsed_docs.rawdocs else []
+        first = lines[0] if lines else ""
+        # Check leading whitespaces in the first non-blank line:
+        # - first line is not blank → use raw docstring first line ('parsed.docs' removed leading spaces)
+        # - first line is blank → use cleaned docstring first line (to find the first non-blank line)
+        if not first.strip() and parsed_docs.docs:
+            lines = parsed_docs.docs.splitlines()
+            first = lines[0] if lines else ""
+        if first and not first.isspace() and leading_ws_regex.search(first):
+            yield cls.DOC012, (first,)
+
+        for section_def in parsed_docs.bad_whitespaces_def:
             yield cls.DOC010, (section_def,)
 
     @classmethod
-    def validate_head_tail_quotes(cls, parsed, /):
-        text = parsed.rawdocs
+    def validate_edge_quotes(cls, parsed_docs, /):
+        text = parsed_docs.rawdocs
         if text:
             lines = text.splitlines()
             if len(lines) > 1 and (quotes_starts_regex.match(lines[0]) or quotes_ends_regex.match(lines[-1])):
@@ -235,17 +322,30 @@ class Violations:
                     yield cls.DOC008, ()  # Summary should end with a period
 
     @classmethod
-    def validate_params(cls, parsed, parameters, /):
-        bag = set()
-        first_raises = min(parsed.raises, key=cls._get_order, default=None)
-        first_return = min(parsed.returns, key=cls._get_order, default=None)
-        for param in parsed.params:
+    def validate_params(cls, parsed_docs, parsed_params, /):
+        """
+        Validate the parameter sections of the parsed docstring against the function definition.
+
+        :param parsed_docs: ParsedDocs parsed: Parsed docstring object.
+        :param dict[str, ParsedFuncParam] parsed_params: Parsed function parameters.
+        :return: Generator yielding violation tuples.
+        """
+
+        documented = list()  # Documented parameters, without duplicates and in order of appearance
+
+        first_raises = min(parsed_docs.raises, key=cls._get_order, default=None)
+        first_return = min(parsed_docs.returns, key=cls._get_order, default=None)
+
+        for param in parsed_docs.params:
             section_key, sep, name, kind, order = param
-            type_hint = parameters.get(name)
-            # Malformed param: missing ':' or missing name/type when required
+
+            param_props = parsed_params.get(name)
+            type_hint = param_props.annotation if param_props else None
+
+            # Malformed: missing sep/name/type(when required)
             if not (sep and name) or (section_key == ptype_key and not kind):
                 yield cls.DOC002, (section_key,)
-            if name and name not in parameters:  # Documented but not in signature
+            if name and name not in parsed_params:  # Documented but not in signature
                 yield cls.DOC101, (name,)
             if kind and not cls.is_valid_type_hint(kind):  # Invalid type syntax
                 yield cls.DOC102, (kind,)
@@ -254,7 +354,7 @@ class Violations:
             if kind and type_hint and kind != type_hint:  # Mismatched type
                 yield cls.DOC104, (kind, type_hint)
 
-            if name and name in bag:  # Duplicated parameter
+            if name and name in documented:  # Duplicated parameter
                 yield cls.DOC105, (name,)
 
             if first_raises and order > first_raises.order:
@@ -264,38 +364,55 @@ class Violations:
                 # Params after returns are considered misplaced
                 yield cls.DOC007, (section_key, first_return.section_key,)
 
-            if name:
-                bag.add(name)
+            if name and name not in documented:
+                documented.append(name)
+
+        # Check parameter order ignoring missing/extra parameters
+        orderby = operator.attrgetter("order")
+        # Ignore parameters not documented
+        defined_documented = [n.name for n in sorted(parsed_params.values(), key=orderby) if n.name in documented]
+        # Ignore documented parameters not in the function definition
+        documented_defined = [n for n in documented if n in parsed_params]
+        if defined_documented != documented_defined:
+            yield cls.DOC106, ()
+
+        # Check for missing parameter keys in docstring
+        ignored = frozenset(("self", "cls", "args", "kwargs"))  # Common ignored parameter names
+        if documented:
+            # Only check for missing parameters if there is at least one documented parameter
+            missing = sorted((parsed_params.keys() - ignored) - frozenset(documented))
+            for name in missing:
+                yield cls.DOC107, (name,)
 
     @classmethod
-    def validate_return(cls, parsed, sign_return_type, has_returns, is_implemented, /):
+    def validate_return(cls, parsed_docs, parsed_return, is_implemented, /):
         """
         Validate the return sections of the parsed docstring.
 
-        :param ParsedDocs parsed: Parsed docstring object.
-        :param str | None sign_return_type: Return type from function signature.
-        :param set[str] has_returns: True if the function has a return or yield statement.
-        :param bool is_implemented:  True if the function is implemented (not just a stub).
-
+        :param ParsedDocs parsed_docs: Parsed docstring object.
+        :param ParsedFuncReturn parsed_return: Parsed function return properties.
+        :param bool is_implemented: True if the function is implemented (not just a stub).
         :return: Generator yielding violation tuples.
         """
 
-        if parsed.returns and (not has_returns and is_implemented):
+        sign_return_type = parsed_return.annotation
+        if parsed_docs.returns and (not parsed_return.has_return and is_implemented):
             yield cls.DOC201, ()  # Return documented but implementation has no return
 
         bag = set()
-        for doc_returns in parsed.returns:
+        for doc_returns in parsed_docs.returns:
             section_key, sep, doc_return_type, description, section_key_ctx, order = doc_returns
 
-            # Malformed return: missing ':' or missing type/description when required or invalid context
+            # Malformed: missing sep/type/description(when required) or invalid context
             if not sep or section_key_ctx:
-                # Missing ':' separator or invalid context (section_key_ctx is True)
+                # Missing sep or invalid context (section_key_ctx is True)
                 yield cls.DOC002, (section_key,)
             elif section_key in return_set and not description:
                 # :return:´ without description
                 yield cls.DOC002, (section_key,)
             elif section_key == rtype_key and not doc_return_type:  # ´:rtype:´ without type
                 yield cls.DOC002, (section_key,)
+
             # Invalid or redundant return type checks
             if doc_return_type and not cls.is_valid_type_hint(doc_return_type):  # Invalid return type
                 yield cls.DOC202, (doc_return_type,)
@@ -306,15 +423,21 @@ class Violations:
             # Duplicate return section
             if section_key in bag:
                 yield cls.DOC205, (section_key,)  # Duplicate return section
-            bag.add(section_key)
+
+            if section_key in return_set:
+                bag.update(return_set)  # Mark all return section as used to avoid duplicates (:return: & :returns:)
+            else:
+                bag.add(section_key)
 
     @classmethod
-    def validate_raises(cls, parsed, /):
-        bag = set()
-        first_return = min(parsed.returns, key=cls._get_order, default=None)
+    def validate_raises(cls, parsed_docs, /):
+        """Validate the raises sections of the parsed docstring."""
 
-        for section_key, sep, error_types, order in parsed.raises:
-            if not (sep and error_types):  # Missing ':' or missing error types
+        bag = set()
+        first_return = min(parsed_docs.returns, key=cls._get_order, default=None)
+
+        for section_key, sep, error_types, order in parsed_docs.raises:
+            if not (sep and error_types):  # Missing sep/error types
                 yield cls.DOC002, (section_key,)
 
             is_invalid = any(not cls.is_valid_syntax(e) for e in error_types)
@@ -331,14 +454,16 @@ class Violations:
                 yield cls.DOC007, (section_key, first_return.section_key,)
 
     @classmethod
-    def validate_variables(cls, parsed, /):
+    def validate_variables(cls, parsed_docs, /):
+        """Validate the variable sections of the parsed docstring."""
+
         bag = set()
-        for var in parsed.variables:
+        for var in parsed_docs.variables:
             section_key, sep, name, kind, with_spaces, description, order = var
             missing_desc = section_key in class_var_set and not description
             missing_type = section_key == vartype_key and not kind
 
-            # Malformed var: missing ':' or missing name/type when required or missing description when required
+            # Malformed: missing sep/name/description(when required)/type(when required)
             if not (sep and name) or missing_type or missing_desc:
                 yield cls.DOC002, (section_key,)
             if kind and not cls.is_valid_type_hint(kind):  # Invalid type syntax
@@ -352,28 +477,22 @@ class Violations:
                 bag.add((section_key, name))
 
     @classmethod
-    def discover_all(cls, parsed, parameters, has_returns, is_implemented, /):
-        if parsed.code_ini_lineno and parsed.docs_end_lineno and parsed.code_ini_lineno - parsed.docs_end_lineno == 1:
-            yield cls.DOC003, ()
-        yield from ((cls.DOC001, (section_key,)) for section_key in parsed.invalid)
+    def discover_all(cls, parsed_docs, parsed_func, /):
+        yield from ((cls.DOC001, (section_key,)) for section_key in parsed_docs.invalid)
+        yield from cls.validate_blank_lines(parsed_docs)
+        yield from cls.validate_whitespaces(parsed_docs)
+        yield from cls.validate_edge_quotes(parsed_docs)
+        yield from cls.validate_summary(parsed_docs)
 
-        yield from cls.validate_empty_lines(parsed)
-        yield from cls.validate_head_tail_quotes(parsed)
-        yield from cls.validate_summary(parsed)
-        yield from cls.validate_bad_whitespaces(parsed)
+        if parsed_docs.kind == NodeTypes.FUNCTION:  # Only functions have parameters, returns, and raises
+            yield from cls.validate_params(parsed_docs, parsed_func.parameters)
+            yield from cls.validate_return(parsed_docs, parsed_func.returns, parsed_func.is_implemented)
+            yield from cls.validate_raises(parsed_docs)
+        elif parsed_docs.kind in (NodeTypes.CLASS, NodeTypes.MODULE):  # Only classes and modules have variables
+            yield from cls.validate_variables(parsed_docs)
 
-        if parsed.non_empty_end_lines:
-            yield cls.DOC011, ()
-
-        if parsed.kind == NodeTypes.FUNCTION:  # Only functions have parameters, returns, and raises
-            yield from cls.validate_params(parsed, parameters)
-            yield from cls.validate_return(parsed, parameters.get("return"), has_returns, is_implemented)
-            yield from cls.validate_raises(parsed)
-        elif parsed.kind in (NodeTypes.CLASS, NodeTypes.MODULE):  # Only classes and modules have variables
-            yield from cls.validate_variables(parsed)
-
-    def discover(self, parsed, parameters, has_returns, is_implemented, /):
-        for (_, code, msg), ctx in self.discover_all(parsed, parameters, has_returns, is_implemented):
+    def discover(self, parsed_docs, parsed_func, /):
+        for (_, code, msg), ctx in self.discover_all(parsed_docs, parsed_func):
             if code in self.valid:
                 self.stats[code] += 1
                 yield ((code, msg), ctx)
@@ -461,12 +580,12 @@ def parse_section_return(section_key, sep, parts_a, parts_b, order, /):
 
 def itersections(docstring, /):
     if docstring:
-        yield from (chunk.strip() for match in section_regex.finditer(docstring) if (chunk := match.group(0)))
+        yield from (chunk for match in section_regex.finditer(docstring) if (chunk := match.group()))
 
 
 def get_summary(docstring, /):
     if docstring and (match := summary_regex.match(docstring)):
-        return match.group(0)
+        return match.group()
     else:
         return None
 
@@ -496,18 +615,17 @@ def fetch_bad_ws_definitions(section, section_key, sep, a_raw, b_raw, /):
     :return: Generator yielding section definitions with bad whitespaces.
     """
 
-    if section_key in types_set:  # Check both sides for type sections
-        # Get right side (after '') without leading spaces and split into lines
-        right_side_lines = b_raw.removeprefix(" ").splitlines()
-        # Only first line of right side is relevant.
-        right_side = right_side_lines[0] if right_side_lines else ""
-        if section_bad_ws_regex.search(a_raw) or section_bad_ws_regex.search(right_side):
-            yield section.splitlines()[0]  # Return only the section definition line
+    if section_key in types_set:
+        if section_ws_left_err(a_raw) or section_ws_right_type_err(b_raw):  # Check left-side and right-side type
+            yield section
 
-    elif section_bad_ws_regex.search(a_raw):  # Other sections only check left-side
+    elif section_ws_left_err(a_raw):
         # Reconstruct section definition with only left-side
         yield ":{}{}".format(a_raw, ":" if sep else "")
 
+    elif section_ws_right_desc_err(b_raw):  # Check right-side description
+        yield section
+
 
 def parse_docs(node, filename, /):
     params = list()
@@ -562,10 +680,10 @@ def parse_docs(node, filename, /):
         elif section_key not in invalid:
             invalid.append(section_key)
 
-    non_empty_end_lines = False
+    non_blank_end_lines = False
     if section_key in types_set and len(b_raw.splitlines()) > 1:
-        # Check for non-empty lines after the last section. (Only for type sections that cannot have a description)
-        non_empty_end_lines = any(line.strip() for line in b_raw.splitlines()[1:])  # Non-empty lines
+        # Check for non-blank lines after the last section. (Only for type sections that cannot have a description)
+        non_blank_end_lines = any(line.strip() for line in b_raw.splitlines()[1:])  # Non-empty lines
 
     if docs:
         summary = get_summary(docs)
@@ -575,12 +693,13 @@ def parse_docs(node, filename, /):
         summary = None
         docs_ini_lineno = docs_end_lineno = None
 
-    code_ini_lineno = None
     if docs_end_lineno:
         after_lineno = docs_end_lineno + 1
+        # Use linecache instead of ast to get the next line after the docstring correctly, because ast skips comments.
         after_line = linecache.getline(filename, after_lineno)
-        if after_line.strip() != "":  # Non-empty line after docstring
-            code_ini_lineno = after_lineno
+        missing_blank_line_after = after_line.strip() != ""
+    else:
+        missing_blank_line_after = False
 
     return ParsedDocs(
         kind=kind,
@@ -589,7 +708,6 @@ def parse_docs(node, filename, /):
         docs=docs,
         docs_ini_lineno=docs_ini_lineno,
         docs_end_lineno=docs_end_lineno,
-        code_ini_lineno=code_ini_lineno,
         params=params,
         returns=returns,
         raises=raises,
@@ -597,7 +715,8 @@ def parse_docs(node, filename, /):
         invalid=invalid,
         ignored=ignored,
         bad_whitespaces_def=bad_whitespaces_def,
-        non_empty_end_lines=non_empty_end_lines,
+        missing_blank_line_after=missing_blank_line_after,
+        non_blank_end_lines=non_blank_end_lines,
     )
 
 
@@ -607,8 +726,7 @@ def has_return_or_yield(node, /):
     contains a 'return', 'yield', or 'yield from' statement in its main body
     (excluding nested functions).
 
-    :param ast.FunctionDef | ast.AsyncFunctionDef node: Root node to explore.
-
+    :param ast.AST node: Root node to explore.
     :return: Whether the function has either "return" or "yield".
     :rtype: bool
     """
@@ -618,25 +736,25 @@ def has_return_or_yield(node, /):
             self.found = False
             self.depth = 0
 
-        def visit_nested(self, node, /):
+        def visit_nested(self, node_, /):
             if self.depth == 0:
                 self.depth += 1
-                self.generic_visit(node)
+                self.generic_visit(node_)
                 self.depth -= 1
 
-        def visit_FunctionDef(self, node, /):
-            self.visit_nested(node)
+        def visit_FunctionDef(self, node_, /):
+            self.visit_nested(node_)
 
-        def visit_AsyncFunctionDef(self, node, /):
-            self.visit_nested(node)
+        def visit_AsyncFunctionDef(self, node_, /):
+            self.visit_nested(node_)
 
-        def visit_Return(self, node, /):
+        def visit_Return(self, node_, /):
             self.found = True
 
-        def visit_Yield(self, node, /):
+        def visit_Yield(self, node_, /):
             self.found = True
 
-        def visit_YieldFrom(self, node, /):
+        def visit_YieldFrom(self, node_, /):
             self.found = True
 
     visitor = ReturnYieldVisitor()
@@ -653,7 +771,7 @@ def is_not_implemented(node, /, rawdocs=None):
       - `...` (Ellipsis literal)
     Otherwise, returns False.
 
-    :param ast.FunctionDef | ast.AsyncFunctionDef node: Root node to explore.
+    :param ast.AST node: Root node to explore.
     :param str | None rawdocs: Raw docstring.
 
     :rtype: bool
@@ -672,11 +790,7 @@ def is_not_implemented(node, /, rawdocs=None):
         elif isinstance(stmt, ast.Raise):  # Case: "raise NotImplementedError()"
             exc = stmt.exc
             error_name = NotImplementedError.__name__
-            if (
-                isinstance(exc, ast.Call)
-                and isinstance(exc.func, ast.Name)
-                and exc.func.id == error_name
-            ):
+            if isinstance(exc, ast.Call) and isinstance(exc.func, ast.Name) and exc.func.id == error_name:
                 return True
 
             else:  # Case: "raise NotImplementedError"
@@ -688,6 +802,38 @@ def is_not_implemented(node, /, rawdocs=None):
             return False
 
 
+def get_args(node, /):
+    """Yield function parameters in the exact order defined by Python's function signature grammar."""
+
+    yield from node.args.posonlyargs  # Positional-only args
+    yield from node.args.args  # Regular args
+    if vararg := node.args.vararg:  # Var positional arg (*args)
+        yield vararg
+    yield from node.args.kwonlyargs  # Keyword-only args
+    if kwarg := node.args.kwarg:  # Var keyword arg (**kwargs)
+        yield kwarg
+
+
+def get_params_props(node, /):
+    """Returns an iterator with each function parameter's properties."""
+
+    for order, arg in enumerate(get_args(node)):
+        annotation = ast.unparse(ann) if (ann := arg.annotation) else None
+        yield (arg.arg, ParsedFuncParam(arg.arg, order, annotation))
+
+
+def get_return_props(node, has_returns, /):
+    """Fetches the function return properties."""
+
+    annotation = ast.unparse(node.returns) if node.returns else None
+    return ParsedFuncReturn(has_returns, annotation)
+
+
+def check_node(node, violations, filename, /):
+    for lineno, code, msg, ctx in checker(node, violations, filename):
+        yield (lineno, code, msg.format(*ctx))
+
+
 def checker(node, violations, filename, /):
     """
     Explore the given AST node and yield each violation found.
@@ -700,41 +846,21 @@ def checker(node, violations, filename, /):
     :rtype: typing.Iterator[tuple[int, str, dict]]
     """
 
-    parsed = parse_docs(node, filename)
-    lineno = parsed.docs_ini_lineno
+    parsed_docs = parse_docs(node, filename)
+    lineno = parsed_docs.docs_ini_lineno
 
-    if parsed.kind == "function":
-        func_params = dict(get_params(node))
-        func_has_returns = has_return_or_yield(node)
-        func_is_implemented = not is_not_implemented(node, rawdocs=parsed.rawdocs)
+    if parsed_docs.kind == NodeTypes.FUNCTION:
+        func_params_props = dict(get_params_props(node))
+        func_return_props = get_return_props(node, has_return_or_yield(node))
+        func_is_implemented = not is_not_implemented(node, rawdocs=parsed_docs.rawdocs)
+        parsed_func = ParsedFuncDef(func_params_props, func_return_props, func_is_implemented)
     else:  # ClassDef or Module
-        func_params = dict()
-        func_has_returns = False
-        func_is_implemented = False
+        parsed_func = None
 
-    for (code, msg), ctx in violations.discover(parsed, func_params, func_has_returns, func_is_implemented):
+    for (code, msg), ctx in violations.discover(parsed_docs, parsed_func):
         yield (lineno, code, msg, ctx)
 
 
-def get_args(node, /):
-    yield from node.args.posonlyargs  # Positional-only args
-    yield from node.args.args  # Regular args
-    yield from node.args.kwonlyargs  # Keyword-only args
-    yield from filter(None, (node.args.vararg, node.args.kwarg))  # *args, **kwargs
-
-
-def get_params(node, /):
-    for arg in get_args(node):
-        yield (arg.arg, ast.unparse(ann) if (ann := arg.annotation) else None)
-    if node.returns:
-        yield ("return", ast.unparse(node.returns))
-
-
-def check_node(node, violations, filename, /):
-    for lineno, code, msg, ctx in checker(node, violations, filename):
-        yield (lineno, code, msg.format(*ctx))
-
-
 def walk_module(quiet, data, filename, /):
     """
     Yields each function node from the parsed "data" tree.
@@ -798,6 +924,55 @@ def dump_file(violations, quiet, path, /):
             print(fmt(lineno, code, msg))
 
 
+def dump_version():
+    path = pathlib.Path(__file__).with_name("pyproject.toml")
+    if path.exists():  # standalone script
+        with path.open("rb") as f:
+            version = tomllib.load(f)["project"]["version"]
+    else:  # installed package
+        version = importlib.metadata.version("sphinx-linter")
+    print("sphinx-linter {}".format(version))
+
+
+def get_config_start_dir(paths, /):
+    resolve = (pathlib.Path(p).resolve() for p in paths)
+    parents = map(str, ((p.parent if p.is_file() else p) for p in resolve))
+    return os.path.commonpath(parents)
+
+
+def load_config(config_file, check_files, /):
+    """
+    Load configuration from 'pyproject.toml' if available.
+
+    If 'config_file' is provided, it is used directly.
+    Otherwise, search upward from the common ancestor of 'check_files' for 'pyproject.toml'
+
+    :param str | None config_file: Path to 'pyproject.toml' file.
+    :param list[str] check_files: List of files/directories to check.
+    :rtype: dict
+    """
+
+    config = dict()
+    if config_file:
+        paths = (pathlib.Path(config_file),)
+    else:
+        dirpath = pathlib.Path(get_config_start_dir(check_files))
+        parents = (dirpath, *dirpath.parents)
+        paths = (p for parent in parents if (p := (parent / "pyproject.toml")) and p.exists())
+
+    for path in paths:
+        try:
+            data = tomllib.loads(path.read_text())
+        except tomllib.TOMLDecodeError:
+            continue  # pyproject is malformed, skip
+        else:
+            if "tool" in data and "sphinx-linter" in data["tool"]:
+                # Only use if it has [tool.sphinx-linter] section
+                config = data["tool"]["sphinx-linter"]
+                break
+    return config
+
+
 def main():
     logging.basicConfig(level=logging.INFO, format='%(levelname)s: %(message)s')
 
@@ -806,27 +981,27 @@ def main():
         "files",
         nargs=argparse.ZERO_OR_MORE,
         default=[os.path.curdir],
-        help="files or directories to check",
+        help="Files or directories to check",
     )
     parser.add_argument(
         "--statistics",
         action="store_const",
         const=True,
         default=False,
-        help="show counts for every rule with at least one violation",
+        help="Show counts for every rule with at least one violation",
     )
     parser.add_argument(
         "--quiet",
         action="store_const",
         const=True,
         default=False,
-        help="print diagnostics, but nothing else",
+        help="Print diagnostics, but nothing else",
     )
     parser.add_argument(
         "--ignore",
         nargs=argparse.ZERO_OR_MORE,
-        default=default_ignore_dirs,
-        help="directories to ignore. Defaults to: {}".format(", ".join(default_ignore_dirs)),
+        default=[],
+        help="Directories to ignore. Defaults to: ({})".format(", ".join(default_ignore_dirs)),
     )
     parser.add_argument(
         "--enable",
@@ -842,13 +1017,47 @@ def main():
         default=[],
         help="Violation codes to disable",
     )
+    parser.add_argument(
+        "--version",
+        action="store_const",
+        const=True,
+        default=False,
+        help="Print version and exit",
+    )
+    parser.add_argument(
+        "--config",
+        type=str,
+        default=None,
+        help=(
+            "Path to 'pyproject.toml' configuration file. "
+            "If not provided, search upward from the files’ common ancestor for 'pyproject.toml'"
+        ),
+    )
+    parser.add_argument(
+        "--isolated",
+        action="store_const",
+        const=True,
+        default=False,
+        help="Run in isolated mode, ignoring configuration files",
+    )
 
     # Disable SyntaxWarnings to reduce output noise from python parser
     #   for example: SyntaxWarning: invalid escape sequence
     warnings.simplefilter("ignore", SyntaxWarning)
     args = parser.parse_args()
-    violations = Violations(enable=args.enable, disable=args.disable)
-    for path in walk(args.files, args.ignore):
+
+    if args.version:
+        dump_version()
+        return 0
+
+    config = dict() if args.isolated else load_config(args.config, args.files)
+    # CLI Arguments have precedence over config file settings
+    enable = args.enable or config.get("enable", [])
+    disable = args.disable or config.get("disable", [])
+    ignore = args.ignore or config.get("ignore", default_ignore_dirs)
+
+    violations = Violations(enable=enable, disable=disable)
+    for path in walk(args.files, ignore):
         dump_file(violations, args.quiet, path)
 
     if args.statistics and violations.stats:
