@@ -58,7 +58,7 @@ def dummy():
         assert not result
 
     @pytest.mark.parametrize("value", (None, ""))
-    def test_validate_empty_lines(self, value):
+    def test_validate_blank_lines(self, value):
         # Test condition when there is no docstring (or empty)
         parsed = sphinxlinter.ParsedDocs(
             kind="function",
@@ -70,14 +70,14 @@ def dummy():
             invalid=list(),
             ignored=list(),
             bad_whitespaces_def=list(),
-            non_empty_end_lines=False,
+            non_blank_end_lines=False,
             rawdocs=value,
             docs=None,
             docs_ini_lineno=None,
             docs_end_lineno=None,
-            code_ini_lineno=0,
+            missing_blank_line_after=False,
         )
-        result = tuple(sphinxlinter.Violations.validate_empty_lines(parsed))
+        result = tuple(sphinxlinter.Violations.validate_blank_lines(parsed))
         assert not result
 
     @pytest.mark.parametrize(
@@ -109,12 +109,12 @@ def dummy():
             invalid=list(),
             ignored=list(),
             bad_whitespaces_def=list(),
-            non_empty_end_lines=False,
+            non_blank_end_lines=False,
             rawdocs=None,
             docs=None,
             docs_ini_lineno=None,
             docs_end_lineno=None,
-            code_ini_lineno=None,
+            missing_blank_line_after=False,
         )
         result = sphinxlinter.parse_docs(ast.parse(content).body[0], "dummy.py")
         assert result == expected
@@ -214,10 +214,8 @@ def dummy():
 
     def test_discover(self):
         # Test code not in valid rules.
-        parsed = object()
-        parameters = object()
-        has_returns = object()
-        is_implemented = object()
+        parsed_docs = object()
+        parsed_func = object()
         obj = sphinxlinter.Violations(enable=(1, 3))
         values = (
             ((True, 1, "msg1"), "ctx1"),
@@ -229,7 +227,7 @@ def dummy():
             ((3, "msg3"), "ctx3"),
         )
         with unittest.mock.patch.object(obj, "discover_all", return_value=values) as discovered:
-            result = tuple(obj.discover(parsed, parameters, has_returns, is_implemented))
+            result = tuple(obj.discover(parsed_docs, parsed_func))
 
-        discovered.assert_called_once_with(parsed, parameters, has_returns, is_implemented)
+        discovered.assert_called_once_with(parsed_docs, parsed_func)
         assert result == expected
